; =========================================================
; Module: Dialogs
; =========================================================
; Description:
;     Module to display pop-up dialogs. These only draw to
;     to the LCD, not plotSScreen, so you can easily use
;     these to save the display, display a dialog, then
;     restore.
; =========================================================

.module Dialogs
; ---------------------------------------------------------
; Draw -> Draws a simple dialog.
; ---------------------------------------------------------
; Inputs:   (l,h) = top left corner coordinate.
;           (e,d) = bottom right corner coordinate.
; Destroys: af, bc, de, hl.
; Remarks:  Writes to saveSScreen.
; ---------------------------------------------------------
Draw
	ld a,(iy+plotFlags)
	push af
	set plotLoc,(iy+plotFlags)

	push hl
	push de

	inc h
	inc l
	dec d
	dec e
	
	bcall(_ClearRect)
	
	pop de
	pop hl
	
	bcall(_DrawRectBorder)
	
	pop af
	ld (iy+plotFlags),a
	ret
	
; ---------------------------------------------------------
; DrawWithTitle -> Draws a dialog with a titlebar.
; ---------------------------------------------------------
; Inputs:   (l,h) = top left corner coordinate.
;           (e,d) = bottom right corner coordinate.
;           bc -> titlebar caption.
; Destroys: af, bc, de, hl.
; Remarks:  Writes to saveSScreen.
; ---------------------------------------------------------
DrawWithTitle
	ld a,(iy+plotFlags)
	push af
	ld a,(iy+textFlags)
	push af
	ld a,(iy+sGrFlags)
	push af

	push bc
	
	push hl
	push de
	call Draw
	pop de
	pop hl
	
	;(l,h)->(e,d)
	ld d,h
	ld a,h
	sub 6
	ld h,a
	push hl
	
	set plotLoc,(iy+plotFlags)
	bcall(_FillRect)
	
	pop hl
	inc l
	ld (penCol),hl
	
	pop hl
	
	set plotLoc,(iy+plotFlags)
	set textInverse,(iy+textFlags)
	res textEraseBelow,(iy+textFlags)
	res textWrite,(iy+sGrFlags)	
	bcall(_VPutS)

	pop af
	ld (iy+sGrFlags),a
	pop af
	ld (iy+textFlags),a
	pop af
	ld (iy+plotFlags),a
	ret
	
; ---------------------------------------------------------
; DrawMessageFromTable -> Draws message from a table.
; ---------------------------------------------------------
; Inputs:   hl -> table.
;           a = message index.
; Destroys: af, bc, de, hl.
; Remarks:  Writes to saveSScreen.
; ---------------------------------------------------------
DrawMessageFromTable
	push bc
	or a
	jr z,FoundMessage
	
	ld b,a

-	ld a,(hl)
	inc hl
	or a
	jr nz,{-}
	djnz {-}
	
FoundMessage
	ld a,(hl)
	dec a
	inc hl
	pop bc
	; Intentional run-on function.

; ---------------------------------------------------------
; DrawMessage -> Draws a message dialog.
; ---------------------------------------------------------
; Inputs:   hl -> message to display on the dialog.
;           a = message type.
;           b = 1 to not wait for keypresses.
; Destroys: af, bc, de, hl.
; Remarks:  Writes to saveSScreen.
; ---------------------------------------------------------
.enum MessageType, Information, Warning, Error
.var ubyte, CurrentMessage
DrawMessage
	push bc
	push hl
	
	ld (CurrentMessage),a
	
	ld b,0
	ld de,saveSScreen+1
-	ld a,(hl)
	or a
	jr z,{+}
	inc b
	ld (de),a
	inc hl
	inc de
	jr {-}
+	ld a,b
	ld (saveSScreen),a
	ld hl,saveSScreen
	bcall(_SStringLength)
	; a = width of the error message (in pixels).
	add a,17
	ld b,a
	ld a,97
	sub b
	srl a
	; a = offset.
	ld (penCol),a
	ld hl,(penCol)
	push hl

	sub 3
	ld l,a
	add a,b
	add a,4
	ld e,a
	ld h,24
	ld d,44
	
	ld bc,ErrorText
	
	ld a,(CurrentMessage)
	or a
	jr z,{+}

	push de
	ld d,a	
-	ld a,(bc)
	inc bc
	or a
	jr nz,{-}
	dec d
	jr nz,{-}
FoundString
	pop de
	
+	
	call DrawWithTitle
		
	pop hl
	ld (penCol),hl
	ld a,27
	ld (penRow),a

	
	ld a,(iy+plotFlags)
	push af
	set plotLoc,(iy+plotFlags)
	res bufferOnly,(iy+plotFlags)
	
	ld a,(CurrentMessage)
	rrca
	rrca
	rrca
	ld l,a
	ld h,0
	ld de,Icons
	add hl,de
	ld de,(penCol)	
	bcall(_DisplayImage)

	pop af
	ld (iy+plotFlags),a

	pop hl
	ld a,(penCol)
	add a,17
	ld (penCol),a
	ld a,(penRow)
	add a,4
	ld (penRow),a
	bcall(_VPutS)

	pop af
	or a
	ret nz

	bcall(_GetCSC)
-	bcall(_GetCSC)
	halt
	cp skEnter
	ret z
	cp sk2nd
	ret z
	cp skClear
	ret z
	jr {-}

Icons
	.db 15, 15 \ .incbmp "Resources/Information.gif"
	.db 15, 15 \ .incbmp "Resources/Warning.gif"
	.db 15, 15 \ .incbmp "Resources/Error.gif"

ErrorText
	.db "Information",0
	.db "Warning",0
	.db "Error",0

.endmodule